library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity Labx3 is
generic(
	TAM_DATA	: integer := 8; -- Numero de bits do caractere enviado para o LCD
	NUM_DATA	: integer := 7; -- Numero de characteres que serão escritos
	--LCD
	DIV_NUM		: integer := 500;	-- 50Mhz/500 = 100KHz = 10us
	BOOT_TIME	: integer := 2000;  -- 2000/100KHz = 20ms
   REFRESH     : integer := 10000000; -- taxa de atualizaçao do LCD, 10000000*20ns = 0,2s => é atualizado a cada 0,2s
	WR_TIME		: integer := 5;	-- 5*10us = 50us
	CLR_TIME   	: integer := 200;	-- 2ms
   DEBOUNCE    : integer := 2500000   -- filtro ruido de trepidaçao, 2500000*20ns = 0,05s
   );
port(
		CLOCK_50MHz		: in std_logic;
		KEY				: in std_logic_vector (11 downto 0);
		SW					: in std_logic_vector (3 downto 0);
      
      --LCD
      LCD_D		   : inout	std_logic_vector(7  downto 0); 	-- LCD data is a bidirectional bus...
      LCD_RS 		: out   std_logic;                		-- LCD register select
      LCD_RW  		: out   std_logic;                		-- LCD Read / nWrite
      LCD_EN  		: out   std_logic;                		-- LCD Enable
      LCD_BACKLIGHT	: out   std_logic
	);
end Labx3;

architecture behavior of Labx3 is

signal w_50mhz_clk            : std_logic;
signal w_reset_50mhz          : std_logic;
signal w_100khz_clk           : std_logic;
signal w_reset_100khz         : std_logic;
signal start_lcd  				: std_logic;
signal key_ready  				: std_logic;
signal data_dec   				: std_logic_vector (7 downto 0);
signal key_data   				: std_logic_vector (3 downto 0);
signal key_wt_11  				: std_logic_vector (11 downto 0);  -- sig das chaves sem key(11), que será o reset
signal idx       					: integer range 1 to NUM_DATA;
signal A,B     				   : std_logic_vector (3 downto 0)  := "0000";
signal result     				: std_logic_vector (7 downto 0);
signal sig							: std_logic_vector (7 downto 0); 
signal count    					: integer range 1 to NUM_DATA + 1 := 1;
signal count_2						: integer range 1 to REFRESH := 1;
signal flag					   	: std_logic := '1';

-- Declaraçao do teclado
component teclado_base is
generic(
	DEBOUNCE : integer
);
port(
		clk_i				:	in std_logic;
		push_button_i	:	in std_logic_vector (11 downto 0);
		key_o				: 	out std_logic_vector (3 downto 0);
		ready_o			:	out std_logic
	);
end component;

-- Declaraçao do LCD
COMPONENT lcd_top
	GENERIC ( TAM_DATA : INTEGER; NUM_DATA : INTEGER; DIV_NUM : INTEGER; BOOT_TIME : INTEGER;
		 WR_TIME : INTEGER; CLR_TIME : INTEGER );
	PORT
	(
		CLK_50MHz_i		:	 IN STD_LOGIC;
		rst_i		   	:	 IN STD_LOGIC;
      start_i			:	 IN STD_LOGIC;
		data_i			:	 IN STD_LOGIC_VECTOR(tam_data DOWNTO 1);
		idx_o			   :	 OUT INTEGER range 1 to NUM_DATA;
		ready_o			:	 OUT STD_LOGIC;
		LCD_DATA			:	 INOUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		LCD_RS			:	 OUT STD_LOGIC;
		LCD_RW			:	 OUT STD_LOGIC;
		LCD_EN			:	 OUT STD_LOGIC;      
		LCD_BLON			:	 OUT STD_LOGIC
	);
END COMPONENT;
	-- Declaração do memoria RAM
	type MEM is array (1 to NUM_DATA) of std_logic_vector(TAM_DATA downto 1);
	signal word: MEM := MEM'(
      x"20", -- Nulo
      x"20",
      x"20",
      x"20",
      x"20",
		x"20",
      x"20"
	);

-- Declaração PLL
COMPONENT my_altpll
   PORT (
		inclk0	 : IN  STD_LOGIC; -- OSC 50MHZ
		c0	       : OUT STD_LOGIC -- 50MHZ
	);
END COMPONENT;

-- Declaração sincronizador de reset
COMPONENT reset_sync
   PORT
	(
		i_clk	               :	 IN STD_LOGIC;
		i_external_reset_n	:	 IN STD_LOGIC;
		o_reset_n      		:	 OUT STD_LOGIC;
		o_reset		         :	 OUT STD_LOGIC
   );
END COMPONENT;   
   
begin

   my_altpll_inst : my_altpll 
   PORT MAP (
		inclk0	=> CLOCK_50MHz,
		c0	      => w_50mhz_clk
	);

   --reset <= KEY(11);
	reset_synch_50mhz_inst : reset_sync
	PORT MAP (
		i_clk                => w_50mhz_clk,
		i_external_reset_n   => KEY(11),
		o_reset		         => open,
		o_reset_n            => w_reset_50mhz
	);
   

   key_wt_11 <= '0' & KEY (10 downto 0);
   
	-- Instanciaçao do teclado	
	teclado:
	teclado_base 
   generic map
   (
      DEBOUNCE => DEBOUNCE
   )
   port map
	(
		clk_i				=> w_50mhz_clk,
		push_button_i 	=> key_wt_11,
		key_o 		   => key_data,	
      ready_o        => key_ready
	);
	
   -- Instanciaçao do LCD
	inst_lcd_top : lcd_top
   generic map(
		TAM_DATA 	=> TAM_DATA,
		NUM_DATA	   => NUM_DATA,
		--LCD
		DIV_NUM		=> DIV_NUM,
		BOOT_TIME	=> BOOT_TIME,
		WR_TIME		=> WR_TIME,
		CLR_TIME	   => CLR_TIME
      )
	port map (
		CLK_50MHz_i	=> w_50mhz_clk,
		rst_i			=> w_reset_50mhz,
      start_i		=> start_lcd,
		data_i		=> data_dec,
		idx_o			=> idx,
		ready_o		=> open,
		-- LCD
		LCD_DATA	=> LCD_D,
		LCD_RS 	=> LCD_RS,
		LCD_RW  	=> LCD_RW,
		LCD_EN  	=> LCD_EN,      
		LCD_BLON => LCD_BACKLIGHT
      
	);
   
   data_dec <= word(idx);
   
	process(w_50mhz_clk, w_reset_50mhz)
	variable C 			: std_logic_vector (3 downto 0);
   variable par_A, par_B, par_C  : integer range 0 to 9;
	begin
		if w_reset_50mhz = '1' then
			result <= (others => '0');
		elsif rising_edge(w_50mhz_clk) then
			par_A := to_integer(unsigned(A));   -- valor parcial A (inteiro)
			par_B := to_integer(unsigned(B));   -- valor parcial B (inteiro)
			if par_B /= 0 then
				par_C := par_A/par_B;            -- conversão para inteiro necessario por causa do operado '/'
			else
				par_C := 0;
			end if;
			C := std_logic_vector(to_unsigned(par_C, 4));   -- conversão de volta pra std_logic
			case sig is
				when x"2B"	=>   result <= "0000" & A + B;	-- operaçao de soma
				when x"2D" 	=>   result <= "0000" & A - B;	-- operação de subtraçao
				when x"2A" 	=>   result <= A * B;				--	operaçao de multiplicaçao
				when x"2F" 	=>   result <= "0000" & C;		-- operaçao de divisão
				when others =>   result <= "00000000";
			end case;
		end if;
	end process;
	
-- Processo para escrever os caracteres desejados no LCD	
process (w_50mhz_clk, w_reset_50mhz)
   begin
   if (w_reset_50mhz = '1') then
      count <= 1;
		word (1) <= x"20";
		word (2) <= x"20";
		word (3) <= x"20";
		word (4) <= x"20";
		word (5) <= x"20";
		word (6) <= x"20";
		word (7) <= x"20";
   elsif rising_edge(w_50mhz_clk) then
      if (KEY(10 downto 0) /= b"000_0000_0000" and flag = '1' ) then
         if key_ready = '1' and key_data /= "1010" then
            if count < (NUM_DATA-2) then
               word(count) <= x"30" + key_data; -- conversão de decimal para ASCII
               -- Dependendo se ja foi adquirido algum numero, o valor do teclado é atribuido a A ou a B
					if count = 1 then
                  A <= key_data;
               elsif count = 3 then
                  B <= key_data;
               end if;
            end if;
				-- Count é usado como o endereço da memória onde será escrito o caractere
            if count = (NUM_DATA - 2) then
               count <= count;
            elsif count <  NUM_DATA-2 then
               count <= count + 2; -- Pula de 2 em 2 pois no meio dos numeros irão os operadores
            end if;
            flag <= '0';
         end if;
      elsif KEY(10 downto 0) = b"000_0000_0000" and flag = '0' then
         flag <= '1';
      end if;
      if count = 3 then
			word(2) <= sig;
      elsif count = 5 or count = 6 then
			count <= count + 1;
		elsif count = 7 then
			word (4) <= x"3D";  -- recebe =
			if result(7) = '0' then
				word (5) <= x"20";
				word (6) <= (x"30" + (to_integer(signed(result))/10)); -- recebe a dezena do resultado
				word (7) <= (x"30" + (to_integer(signed(result))rem 10));  -- recebe a unidade do resultado
			else
				word (5) <= x"2D";
				word (6) <= (x"30" - (to_integer(signed(result))/10)); -- recebe a dezena do resultado
				word (7) <= (x"30" - (to_integer(signed(result))rem 10));  -- recebe a unidade do resultado
			end if;
			count <= count + 1;
      end if;
	end if;
   end process;
	
	sig <= x"2B" when SW(0) = '1' else -- recebe o caractere +
          x"2D" when SW(1) = '1' else -- recebe o caractere -
          x"2A" when SW(2) = '1' else -- recebe o caractere *
          x"2F" when SW(3) = '1' else -- recebe o caractere /
          x"00";
	
   -- Processo para atualizar o display LCD
   process (w_50mhz_clk, w_reset_50mhz)
   begin
   if (rising_edge(w_50mhz_clk) and w_reset_50mhz /= '1') then
      start_lcd <= '0';
		count_2 <= count_2 + 1;
      if count_2 = (REFRESH - 4) then -- -4, tempo para o sig ser interpretado sem problemas
         start_lcd <= '1';
      elsif count_2 = (REFRESH) then 
         count_2 <= 1;
      end if;
   end if;
   end process;
   
end behavior;